var gl, ctx;
var tempCanvas, globalCanvas;
var projectionMatrix;

var UserUpdate, UserDraw;

function EngineMain(canvas, loadCB, updateCB, drawCB) {
    globalCanvas = canvas;
    InitGL(canvas);
    InitTempCanvas();
    
    InitSpriteParams();
    SetupTextRendering();
	SetupInput();
    
    if(loadCB) loadCB();
    
    projectionMatrix = mat4.create();
    mat4.ortho(projectionMatrix, 0, canvas.width, canvas.height, 0, -1000, 1000);
    
    if(!updateCB) {
        console.error("You need to specify an update function!");
        return;
    }
    
    if(!drawCB) {
        console.error("You need to specify a draw function!");
        return;
    }
    
    UserUpdate = updateCB;
    UserDraw = drawCB;
    
    var loop = setInterval(Loop, 15);
}

function InitGL(canvas) {
    try {
        gl = canvas.getContext("experimental-webgl");
        gl.viewportWidth = canvas.width;        
        gl.viewportHeight = canvas.height;
        gl.enable(gl.BLEND)
        gl.blendFunc(gl.SRC_ALPHA, gl.ONE);
        gl.clearColor(1, 0, 0, 1);
    } catch(e) {
    }
    
    if(!gl) {
        alert("WebGL failed to load.");    
    }    
}

function InitTempCanvas() {
    tempCanvas = document.createElement('canvas');
    tempCanvas.id = "TempCanvas";
    tempCanvas.width = 1000;
    tempCanvas.height = 1000;
    document.body.appendChild(tempCanvas);
    tempCanvas.style.visibility = "hidden";
    ctx = tempCanvas.getContext("2d");
    
    if(!tempCanvas) console.log("Temp canvas failed!"); 
    if(!ctx) console.log("2D context failed!");
}

function Loop() {
    Update((1 / 1000) * 16.666667);    
    Draw((1 / 1000) * 16.666667);
}

function Update(dt) {
    UserUpdate(dt);
}

function Draw(dt) {
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
    UserDraw(dt);
}