var SPRITE_VERTICES = [
    0.5, 0.5, 0, 1, 1,
    -0.5, 0.5, 0, 0, 1,
    0.5, -0.5, 0, 1, 0,
    -0.5, -0.5, 0, 0, 0
];

var SPRITE_BUFFER;
var SPRITE_CURRENT_SHADER;

function InitSpriteParams() {
    SPRITE_BUFFER = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, SPRITE_BUFFER);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(SPRITE_VERTICES), gl.STATIC_DRAW);
    SPRITE_BUFFER.vertexSize = 3;
    SPRITE_BUFFER.texSize = 2;
    SPRITE_BUFFER.numItems = 4;
}

function Sprite(texture, color) {
    if(!texture) {
        this.texture = null;
    } else {
        this.texture = texture;
    }
    
    if(!color) color = [0, 0, 0, 0];
    this.color = color;
}

var SpriteSetShader = function(shader) {
    SPRITE_CURRENT_SHADER = shader;
    gl.bindBuffer(gl.ARRAY_BUFFER, SPRITE_BUFFER);
    
    SPRITE_CURRENT_SHADER.Bind();
    gl.enableVertexAttribArray(SPRITE_CURRENT_SHADER.positionAttribute);
    gl.enableVertexAttribArray(SPRITE_CURRENT_SHADER.texCoordAttribute);
    gl.vertexAttribPointer(SPRITE_CURRENT_SHADER.positionAttribute, SPRITE_BUFFER.vertexSize, gl.FLOAT, false, 20, 0);            
    gl.vertexAttribPointer(SPRITE_CURRENT_SHADER.texCoordAttribute, SPRITE_BUFFER.texSize, gl.FLOAT, false, 20, 12);

    SPRITE_CURRENT_SHADER.UniformMatrix(SPRITE_CURRENT_SHADER.projectionUniform, projectionMatrix);
}

Sprite.prototype.Draw = function(x, y, scaleX, scaleY, rot) {
    if(!scaleY) scaleY = scaleX;
    
    var modelMat = mat4.create();
    mat4.translate(modelMat, modelMat, [x, y, 0]);
    mat4.scale(modelMat, modelMat, [scaleX, scaleY, 1]);
    
    if(rot) {
        mat4.rotate(modelMat, modelMat, ToRadian(rot), [0, 0, 1]);
    }
    
    if(this.texture) {
        gl.activeTexture(gl.TEXTURE0);
        this.texture.Bind();
        SPRITE_CURRENT_SHADER.UniformTexture(SPRITE_CURRENT_SHADER.textureUniform, 0);
    }
    
    SPRITE_CURRENT_SHADER.UniformMatrix(SPRITE_CURRENT_SHADER.modelviewUniform, modelMat);
    
    if(SPRITE_CURRENT_SHADER.colourUniform != -1) {
        SPRITE_CURRENT_SHADER.UniformFloat4(SPRITE_CURRENT_SHADER.colourUniform, this.color);
    }
    
    gl.drawArrays(gl.TRIANGLE_STRIP, 0, SPRITE_BUFFER.numItems);
    
    if(this.texture) this.texture.Unbind();
    
}
